#!/usr/bin/env python
# Simple example showing a TDDFT calculation

import sys

from qcelemental.models import AtomicInput, Molecule
from tcpb import TCFrontEndClient as TCPBClient

# Ethene system
atoms = ["C", "C", "H", "H", "H", "H"]
geom = [
    0.35673483,
    -0.05087227,
    -0.47786734,
    1.61445821,
    -0.06684947,
    -0.02916681,
    -0.14997206,
    0.87780529,
    -0.62680155,
    -0.16786485,
    -0.95561368,
    -0.69426370,
    2.15270896,
    0.84221076,
    0.19314809,
    2.16553127,
    -0.97886933,
    0.15232587,
]

if len(sys.argv) != 3:
    print("Usage: {} host port".format(sys.argv[0]))
    exit(1)

molecule = Molecule(symbols=atoms, geometry=geom)
atomic_input = AtomicInput(
    molecule=molecule,
    driver="gradient",
    model={
        "method": "wpbeh",
        "basis": "6-31g",
    },
    keywords={
        "cis": "yes",
        "cistarget": 1,
        "cisnumstates": 3,
        "closed_shell": True,
        "restricted": True,
        "rc_w": 0.2,
        "cis": "yes",
        "cistarget": 1,
        "cisnumstates": 3,
        "cisrelaxdipole": "yes",
    },
    protocols={"wavefunction": "all", "native_files": "all"},
)
with TCPBClient(host=sys.argv[1], port=int(sys.argv[2])) as TC:

    # Gradient calculation
    result = TC.compute(atomic_input)
    print("Grad Results:\n{}".format(result))

    # Coupling calculation
    # nac_options = tddft_options.copy()
    # nac_options["nacstate1"] = 0
    # nac_options["nacstate2"] = 1
    # # Try to seed as much guess as possible
    # # TODO: Update this one CI and Z vectors are passed through properly
    # nac_options["guess"] = "{}/{}".format(grad_results["job_scr_dir"], "c0")

    # nac_results = TC.compute_job_sync("coupling", geom, "angstrom", **nac_options)
    # print("\nNAC Results:\n{}".format(nac_results))



#!/usr/bin/env python
# Basic energy calculation
import sys

from qcelemental.models import AtomicInput, Molecule

from tcpb import TCFrontEndClient
from tcpb.config import TCFEKeywords, settings

if len(sys.argv) != 3:
    print(f"Usage: {sys.argv[0]} host port")
    exit(1)


# Water system
atoms = ["O", "H", "H"]
geom = [0.0, 0.0, 0.0, 0.0, 1.5, 0.0, 0.0, 0.0, 1.5]  # in bohr

molecule = Molecule(symbols=atoms, geometry=geom)
atomic_input = AtomicInput(
    molecule=molecule,
    model={
        "method": "b3lyp",
        "basis": "6-31g",
    },
    driver="energy",
    keywords={"restricted": False},
    # TCFrontEndClient can collect stdout
    # TCFrontEnd can also collect native_files produced by TeraCHem from the computation
    protocols={"wavefunction": "all", "stdout": True, "native_files": "all"},
    # TCFrontEndClient will delete scratch unless scratch_messy: True
    extras={settings.tcfe_keywords: {TCFEKeywords.scratch_messy: False}},
)

with TCFrontEndClient(host=sys.argv[1], port=int(sys.argv[2])) as client:
    result = client.compute(atomic_input)

# NOTE: Addition of stdout field possible with TCFrontendClient
print(result.stdout)
print(result)
print(result.return_result)
print(result.extras["job_extras"])
# native_files will contain orb1a/b files in binary form
print(result.native_files.keys())



from qcelemental.models import FailedOperation

from tcpb.clients import TCProtobufClient
from tcpb.exceptions import ServerError


def test_comptue_failed_operation(mocker, atomic_input):
    client = TCProtobufClient()
    # Cause _send_msg to raise ServerError
    patch = mocker.patch("tcpb.clients.TCProtobufClient._send_msg")
    patch.side_effect = ServerError("my message", client)
    # Try computation
    result = client.compute(atomic_input)
    assert isinstance(result, FailedOperation)
    assert result.error.error_type == "terachem_server_error"



from pathlib import Path
from typing import Collection, Union

import pytest
from qcelemental.models import (
    AtomicInput,
    AtomicResult,
    AtomicResultProperties,
    Molecule,
)
from qcelemental.models.common_models import Model

from tcpb import terachem_server_pb2 as pb
from tcpb.config import settings as tcpb_settings


@pytest.fixture
def settings():
    yield {"tcpb_host": "localhost", "tcpb_port": 11111, "round_decimals": 6}


@pytest.fixture
def test_data_dir():
    """Test data directory Path"""
    return Path(__file__).parent / "test_data"


@pytest.fixture
def ethylene():
    # NOTE: Geometry in angstroms
    yield {
        "atoms": ["C", "C", "H", "H", "H", "H"],
        "geometry": [
            0.35673483,
            -0.05087227,
            -0.47786734,
            1.61445821,
            -0.06684947,
            -0.02916681,
            -0.14997206,
            0.87780529,
            -0.62680155,
            -0.16786485,
            -0.95561368,
            -0.69426370,
            2.15270896,
            0.84221076,
            0.19314809,
            2.16553127,
            -0.97886933,
            0.15232587,
        ],
    }


@pytest.fixture(scope="function")
def water():
    return Molecule(
        symbols=["O", "H", "H"],
        geometry=[0.0, 0.0, 0.0, 0.0, 1.5, 0.0, 0.0, 0.0, 1.5],  # in bohr
    )


@pytest.fixture(scope="function")
def atomic_input(water):
    model = Model(method="B3LYP", basis="6-31g")
    return AtomicInput(molecule=water, model=model, driver="energy")


@pytest.fixture(scope="function")
def atomic_result(atomic_input):
    result = AtomicResult(
        **atomic_input.dict(),
        return_result=123.123,
        success=True,
        properties=AtomicResultProperties(return_energy=123.123),
    )
    result.extras[tcpb_settings.extras_job_kwarg] = {"job_dir": "job_1"}
    return result


@pytest.fixture
def job_output():
    """Return job_output protobuf message"""
    job_output_correct_answer = pb.JobOutput()
    with open(
        Path(__file__).parent / "answers" / "cisno_casci_result.pbmsg", "rb"
    ) as f:
        job_output_correct_answer.ParseFromString(f.read())
    return job_output_correct_answer


def _round(value: Union[Collection[float], float], places: int = 6):
    """Round a value or Collection of values to a set precision"""
    if isinstance(value, (float, int)):
        return round(value, places)
    elif isinstance(value, Collection):
        return [_round(v, places) for v in value]
    else:
        raise ValueError(f"Cannot round value of type {type(value)}")



from typing import Any, Dict

from google.protobuf.json_format import MessageToDict
from numpy import array
from qcelemental import Datum
from qcelemental.models import AtomicInput, BasisSet, Molecule
from qcelemental.models.results import AtomicResultProperties, WavefunctionProperties

from . import terachem_server_pb2 as pb
from .config import TCFEKeywords

SUPPORTED_DRIVERS = {"ENERGY", "GRADIENT"}


def atomic_input_to_job_input(atomic_input: AtomicInput) -> pb.JobInput:
    """Convert AtomicInput to JobInput"""
    # Don't mutate original atomic_input object
    ai_copy = atomic_input.copy(deep=True)

    # Create Mol instance
    mol_msg = pb.Mol()
    mol_msg.atoms.extend(ai_copy.molecule.symbols)
    mol_msg.xyz.extend(ai_copy.molecule.geometry.flatten())
    mol_msg.units = pb.Mol.UnitType.BOHR  # Molecule always in bohr
    mol_msg.charge = int(ai_copy.molecule.molecular_charge)
    mol_msg.multiplicity = ai_copy.molecule.molecular_multiplicity
    mol_msg.closed = ai_copy.keywords.pop("closed_shell", True)
    mol_msg.restricted = ai_copy.keywords.pop("restricted", True)
    # Drop keyword terms already applied from Molecule object
    ai_copy.keywords.pop("charge", None)  # mol_msg.charge
    ai_copy.keywords.pop("spinmult", None)  # mol_msg.multiplicity

    # Create JobInput message
    ji = pb.JobInput(mol=mol_msg)
    # Set driver
    driver = ai_copy.driver.upper()
    if driver not in SUPPORTED_DRIVERS:
        # Only support QCEngine supported drivers; energy, gradient, hessian, properties
        raise ValueError(
            f"Driver '{driver}' not supported, please select from {SUPPORTED_DRIVERS}"
        )
    ji.run = pb.JobInput.RunType.Value(driver)
    # Set Method
    ji.method = pb.JobInput.MethodType.Value(ai_copy.model.method.upper())
    # Set Basis
    ji.basis = ai_copy.model.basis

    # Get keywords that have specific protobuf fields
    ji.return_bond_order = ai_copy.keywords.pop("bond_order", False)
    ji.orb1afile = ai_copy.keywords.pop("orb1afile", "")
    ji.orb1bfile = ai_copy.keywords.pop("orb1bfile", "")

    # Request AO and MO information
    if ai_copy.keywords.pop("mo_output", False):
        ji.imd_orbital_type = pb.JobInput.ImdOrbitalType.WHOLE_C

    # Set all other keywords under the "user_options" catch all
    for key, value in ai_copy.keywords.items():
        ji.user_options.extend([key, str(value)])

    return ji


def mol_to_molecule(mol: pb.Mol) -> Molecule:
    """Convert mol protobuf message to Molecule

    Note:
        Should not use for returning AtomicResults objects because the AtomicResult
        object should be a direct superset of the AtomicInput that created it (and
        already contains the Molecule submitted by the user)
    """
    if mol.units == pb.Mol.UnitType.ANGSTROM:
        geom_angstrom = Datum("geometry", "angstrom", array(mol.xyz))
        geom_bohr = geom_angstrom.to_units("bohr")
    elif mol.units == pb.Mol.UnitType.BOHR:
        geom_bohr = array(mol.xyz)
    else:
        raise ValueError(f"Unknown Unit Type: {mol.units} for molecular geometry")
    return Molecule(
        symbols=mol.atoms,
        geometry=geom_bohr,
        molecular_multiplicity=mol.multiplicity,
    )


# def job_output_to_atomic_result(
#     *,
#     atomic_input: AtomicInput,
#     job_output: pb.JobOutput,
#     creator: str,
# ) -> AtomicResult:
#     """Convert JobOutput to AtomicResult"""
#     # Convert job_output to python types
#     # NOTE: Required so that AtomicResult is JSON serializable. Protobuf types are not.
#     jo_dict = MessageToDict(job_output, preserving_proto_field_name=True)

#     if atomic_input.driver.upper() == "ENERGY":
#         # Select first element in list (ground state); may need to modify for excited
#         # states
#         return_result: Union[float, List[float]] = jo_dict["energy"][0]

#     elif atomic_input.driver.upper() == "GRADIENT":
#         return_result = jo_dict["gradient"]

#     else:
#         raise ValueError(
#             f"Unsupported driver: {atomic_input.driver.upper()}, supported drivers "
#             f"include: {SUPPORTED_DRIVERS}"
#         )

#     # Prepare AtomicInput to be base input for AtomicResult
#     atomic_input_dict = atomic_input.dict()
#     atomic_input_dict.pop("provenance", None)

#     # Create AtomicResult as superset of AtomicInput values
#     atomic_result = AtomicResult(
#         **atomic_input_dict,
#         # Create new provenance object
#         provenance=Provenance(
#             creator=creator,
#             version="",
#             routine="tcpb.TCProtobufClient | TCFrontEndClient.compute",
#         ),
#         return_result=return_result,
#         properties=to_atomic_result_properties(job_output),
#         # NOTE: Wavefunction will only be added if atomic_input.protocols.wavefunction != 'none'
#         wavefunction=to_wavefunction_properties(job_output, atomic_input),
#         success=True,
#     )
#     # And extend extras to include values additional to input extras
#     atomic_result.extras.update(
#         {
#             settings.extras_qcvars_kwarg: {
#                 "charges": jo_dict.get("charges"),
#                 "spins": jo_dict.get("spins"),
#                 "meyer_bond_order": jo_dict.get("bond_order"),
#                 "orb_size": jo_dict.get("orb_size"),
#                 "excited_state_energies": jo_dict.get("energy"),
#                 "cis_transition_dipoles": jo_dict.get("cis_transition_dipoles"),
#                 "compressed_bond_order": jo_dict.get("compressed_bond_order"),
#                 "compressed_hessian": jo_dict.get("compressed_hessian"),
#                 "compressed_ao_data": jo_dict.get("compressed_ao_data"),
#                 "compressed_primitive_data": jo_dict.get("compressed_primitive_data"),
#                 "compressed_mo_vector": jo_dict.get("compressed_mo_vector"),
#                 "imd_mmatom_gradient": jo_dict.get("imd_mmatom_gradient"),
#             },
#             settings.extras_job_kwarg: {
#                 "job_dir": jo_dict.get("job_dir"),
#                 "job_scr_dir": jo_dict.get("job_scr_dir"),
#                 "server_job_id": jo_dict.get("server_job_id"),
#                 "orb1afile": jo_dict.get("orb1afile"),
#                 "orb1bfile": jo_dict.get("orb1bfile"),
#             },
#         }
#     )
#     return atomic_result


def to_atomic_result_properties(job_output: pb.JobOutput) -> AtomicResultProperties:
    """Extract AtomicResultProperties from JobOutput protobuf message"""
    return AtomicResultProperties(
        return_energy=job_output.energy[0],
        scf_dipole_moment=job_output.dipoles[
            :-1
        ],  # Cutting out |D| value; see .proto note re: diples
        calcinfo_natom=len(job_output.mol.atoms),
        calcinfo_nmo=len(job_output.orba_energies),
        calcinfo_nalpha=sum(job_output.orba_occupations),
        calcinfo_nbeta=sum(job_output.orbb_occupations),
    )


def to_wavefunction_properties(
    job_output: pb.JobOutput, atomic_input: AtomicInput
) -> WavefunctionProperties:
    """Extract WavefunctionProperties from JobOutput protobuf message"""
    jo_dict = MessageToDict(job_output, preserving_proto_field_name=True)
    return WavefunctionProperties(
        basis=BasisSet(
            name=atomic_input.model.basis,
            center_data={},  # TODO: need to fill out
            atom_map=[],  # TODO: need to fill out
        ),
        restricted=atomic_input.keywords.get("restricted", True),
        scf_eigenvalues_a=jo_dict.get("orba_energies"),
        scf_occupations_a=jo_dict.get("orba_occupations"),
        scf_eigenvalues_b=jo_dict.get("orbb_energies", []),
        scf_occupations_b=jo_dict.get("orbb_occupations", []),
    )


def _validate_tcfe_keywords(tcfe_keywords: Dict[str, Any]) -> None:
    """Validates tcfe:keywords"""
    for key in tcfe_keywords.keys():
        TCFEKeywords(key)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor = ...

global___MessageType = MessageType

class _MessageType(
    google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MessageType.V],
    builtins.type,
):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
    STATUS = MessageType.V(0)
    MOL = MessageType.V(1)
    JOBINPUT = MessageType.V(2)
    JOBOUTPUT = MessageType.V(3)

class MessageType(metaclass=_MessageType):
    V = typing.NewType("V", builtins.int)

STATUS = MessageType.V(0)
MOL = MessageType.V(1)
JOBINPUT = MessageType.V(2)
JOBOUTPUT = MessageType.V(3)

class Status(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    BUSY_FIELD_NUMBER: builtins.int
    ACCEPTED_FIELD_NUMBER: builtins.int
    WORKING_FIELD_NUMBER: builtins.int
    COMPLETED_FIELD_NUMBER: builtins.int
    JOB_DIR_FIELD_NUMBER: builtins.int
    JOB_SCR_DIR_FIELD_NUMBER: builtins.int
    SERVER_JOB_ID_FIELD_NUMBER: builtins.int
    busy: builtins.bool = ...
    accepted: builtins.bool = ...
    working: builtins.bool = ...
    completed: builtins.bool = ...
    job_dir: typing.Text = ...
    job_scr_dir: typing.Text = ...
    server_job_id: builtins.int = ...
    def __init__(
        self,
        *,
        busy: builtins.bool = ...,
        accepted: builtins.bool = ...,
        working: builtins.bool = ...,
        completed: builtins.bool = ...,
        job_dir: typing.Text = ...,
        job_scr_dir: typing.Text = ...,
        server_job_id: builtins.int = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing_extensions.Literal[
            "accepted",
            b"accepted",
            "completed",
            b"completed",
            "job_status",
            b"job_status",
            "working",
            b"working",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "accepted",
            b"accepted",
            "busy",
            b"busy",
            "completed",
            b"completed",
            "job_dir",
            b"job_dir",
            "job_scr_dir",
            b"job_scr_dir",
            "job_status",
            b"job_status",
            "server_job_id",
            b"server_job_id",
            "working",
            b"working",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing_extensions.Literal["job_status", b"job_status"]
    ) -> typing_extensions.Literal["accepted", "working", "completed"]: ...

global___Status = Status

class Mol(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    class _UnitType(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[UnitType.V],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        ANGSTROM = Mol.UnitType.V(0)
        BOHR = Mol.UnitType.V(1)

    class UnitType(metaclass=_UnitType):
        V = typing.NewType("V", builtins.int)
    ANGSTROM = Mol.UnitType.V(0)
    BOHR = Mol.UnitType.V(1)

    ATOMS_FIELD_NUMBER: builtins.int
    XYZ_FIELD_NUMBER: builtins.int
    UNITS_FIELD_NUMBER: builtins.int
    CHARGE_FIELD_NUMBER: builtins.int
    MULTIPLICITY_FIELD_NUMBER: builtins.int
    CLOSED_FIELD_NUMBER: builtins.int
    RESTRICTED_FIELD_NUMBER: builtins.int
    atoms: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        typing.Text
    ] = ...
    xyz: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    units: global___Mol.UnitType.V = ...
    charge: builtins.int = ...
    multiplicity: builtins.int = ...
    closed: builtins.bool = ...
    restricted: builtins.bool = ...
    def __init__(
        self,
        *,
        atoms: typing.Optional[typing.Iterable[typing.Text]] = ...,
        xyz: typing.Optional[typing.Iterable[builtins.float]] = ...,
        units: global___Mol.UnitType.V = ...,
        charge: builtins.int = ...,
        multiplicity: builtins.int = ...,
        closed: builtins.bool = ...,
        restricted: builtins.bool = ...,
    ) -> None: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "atoms",
            b"atoms",
            "charge",
            b"charge",
            "closed",
            b"closed",
            "multiplicity",
            b"multiplicity",
            "restricted",
            b"restricted",
            "units",
            b"units",
            "xyz",
            b"xyz",
        ],
    ) -> None: ...

global___Mol = Mol

class JobInput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...

    class _RunType(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[RunType.V],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        ENERGY = JobInput.RunType.V(0)
        GRADIENT = JobInput.RunType.V(1)
        COUPLING = JobInput.RunType.V(14)
        TDCI = JobInput.RunType.V(16)
        CI_VEC_OVERLAP = JobInput.RunType.V(19)

    class RunType(metaclass=_RunType):
        V = typing.NewType("V", builtins.int)
    ENERGY = JobInput.RunType.V(0)
    GRADIENT = JobInput.RunType.V(1)
    COUPLING = JobInput.RunType.V(14)
    TDCI = JobInput.RunType.V(16)
    CI_VEC_OVERLAP = JobInput.RunType.V(19)

    class _MethodType(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[MethodType.V],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        HF = JobInput.MethodType.V(0)
        CASE = JobInput.MethodType.V(2)
        SVWN1 = JobInput.MethodType.V(3)
        SVWN3 = JobInput.MethodType.V(4)
        SVWN5 = JobInput.MethodType.V(5)
        SVWN = JobInput.MethodType.V(5)
        B3LYP1 = JobInput.MethodType.V(6)
        B3LYP = JobInput.MethodType.V(6)
        B3LYP3 = JobInput.MethodType.V(7)
        B3LYP5 = JobInput.MethodType.V(8)
        BLYP = JobInput.MethodType.V(9)
        BHANDHLYP = JobInput.MethodType.V(10)
        PBE = JobInput.MethodType.V(11)
        REVPBE = JobInput.MethodType.V(12)
        PBE0 = JobInput.MethodType.V(13)
        REVPBE0 = JobInput.MethodType.V(14)
        WPBE = JobInput.MethodType.V(15)
        WPBEH = JobInput.MethodType.V(16)
        BOP = JobInput.MethodType.V(17)
        MUBOP = JobInput.MethodType.V(18)
        CAMB3LYP = JobInput.MethodType.V(19)
        B97 = JobInput.MethodType.V(20)
        WB97 = JobInput.MethodType.V(21)
        WB97X = JobInput.MethodType.V(22)
        WB97XD3 = JobInput.MethodType.V(23)
        GFNXTB = JobInput.MethodType.V(24)
        GFN2XTB = JobInput.MethodType.V(25)

    class MethodType(metaclass=_MethodType):
        V = typing.NewType("V", builtins.int)
    HF = JobInput.MethodType.V(0)
    CASE = JobInput.MethodType.V(2)
    SVWN1 = JobInput.MethodType.V(3)
    SVWN3 = JobInput.MethodType.V(4)
    SVWN5 = JobInput.MethodType.V(5)
    SVWN = JobInput.MethodType.V(5)
    B3LYP1 = JobInput.MethodType.V(6)
    B3LYP = JobInput.MethodType.V(6)
    B3LYP3 = JobInput.MethodType.V(7)
    B3LYP5 = JobInput.MethodType.V(8)
    BLYP = JobInput.MethodType.V(9)
    BHANDHLYP = JobInput.MethodType.V(10)
    PBE = JobInput.MethodType.V(11)
    REVPBE = JobInput.MethodType.V(12)
    PBE0 = JobInput.MethodType.V(13)
    REVPBE0 = JobInput.MethodType.V(14)
    WPBE = JobInput.MethodType.V(15)
    WPBEH = JobInput.MethodType.V(16)
    BOP = JobInput.MethodType.V(17)
    MUBOP = JobInput.MethodType.V(18)
    CAMB3LYP = JobInput.MethodType.V(19)
    B97 = JobInput.MethodType.V(20)
    WB97 = JobInput.MethodType.V(21)
    WB97X = JobInput.MethodType.V(22)
    WB97XD3 = JobInput.MethodType.V(23)
    GFNXTB = JobInput.MethodType.V(24)
    GFN2XTB = JobInput.MethodType.V(25)

    class _ImdType(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ImdType.V],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        NOT_IMD = JobInput.ImdType.V(0)
        IMD_NEW_CONDITION = JobInput.ImdType.V(1)
        IMD_CONTINUE = JobInput.ImdType.V(2)
        IMD_HESSIAN = JobInput.ImdType.V(3)

    class ImdType(metaclass=_ImdType):
        V = typing.NewType("V", builtins.int)
    NOT_IMD = JobInput.ImdType.V(0)
    IMD_NEW_CONDITION = JobInput.ImdType.V(1)
    IMD_CONTINUE = JobInput.ImdType.V(2)
    IMD_HESSIAN = JobInput.ImdType.V(3)

    class _ImdOrbitalType(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ImdOrbitalType.V],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        NO_ORBITAL = JobInput.ImdOrbitalType.V(0)
        ALPHA_ORBITAL = JobInput.ImdOrbitalType.V(1)
        BETA_ORBITAL = JobInput.ImdOrbitalType.V(2)
        ALPHA_DENSITY = JobInput.ImdOrbitalType.V(3)
        BETA_DENSITY = JobInput.ImdOrbitalType.V(4)
        WHOLE_C = JobInput.ImdOrbitalType.V(5)

    class ImdOrbitalType(metaclass=_ImdOrbitalType):
        V = typing.NewType("V", builtins.int)
    NO_ORBITAL = JobInput.ImdOrbitalType.V(0)
    ALPHA_ORBITAL = JobInput.ImdOrbitalType.V(1)
    BETA_ORBITAL = JobInput.ImdOrbitalType.V(2)
    ALPHA_DENSITY = JobInput.ImdOrbitalType.V(3)
    BETA_DENSITY = JobInput.ImdOrbitalType.V(4)
    WHOLE_C = JobInput.ImdOrbitalType.V(5)

    class _ImdAdditionalOption(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[
            ImdAdditionalOption.V
        ],
        builtins.type,
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor = ...
        IMD_NORMAL_MD = JobInput.ImdAdditionalOption.V(0)
        IMD_MECI_OPT_GRADIENT = JobInput.ImdAdditionalOption.V(1)

    class ImdAdditionalOption(metaclass=_ImdAdditionalOption):
        V = typing.NewType("V", builtins.int)
    IMD_NORMAL_MD = JobInput.ImdAdditionalOption.V(0)
    IMD_MECI_OPT_GRADIENT = JobInput.ImdAdditionalOption.V(1)

    MOL_FIELD_NUMBER: builtins.int
    RUN_FIELD_NUMBER: builtins.int
    METHOD_FIELD_NUMBER: builtins.int
    BASIS_FIELD_NUMBER: builtins.int
    USER_OPTIONS_FIELD_NUMBER: builtins.int
    ORB1AFILE_FIELD_NUMBER: builtins.int
    ORB1BFILE_FIELD_NUMBER: builtins.int
    RETURN_BOND_ORDER_FIELD_NUMBER: builtins.int
    XYZ2_FIELD_NUMBER: builtins.int
    IMD_TYPE_FIELD_NUMBER: builtins.int
    IMD_INITIAL_ORBITAL_FIELD_NUMBER: builtins.int
    IMD_ORBITAL_TYPE_FIELD_NUMBER: builtins.int
    IMD_XYZ_PREVIOUS_FIELD_NUMBER: builtins.int
    IMD_MO_PREVIOUS_FIELD_NUMBER: builtins.int
    IMD_MMATOM_POSITION_FIELD_NUMBER: builtins.int
    IMD_MMATOM_INFO_FIELD_NUMBER: builtins.int
    IMD_ADDITIONAL_OPTION_FIELD_NUMBER: builtins.int
    run: global___JobInput.RunType.V = ...
    method: global___JobInput.MethodType.V = ...
    basis: typing.Text = ...
    user_options: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        typing.Text
    ] = ...
    orb1afile: typing.Text = ...
    orb1bfile: typing.Text = ...
    return_bond_order: builtins.bool = ...
    xyz2: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_type: global___JobInput.ImdType.V = ...
    imd_initial_orbital: builtins.int = ...
    imd_orbital_type: global___JobInput.ImdOrbitalType.V = ...
    imd_xyz_previous: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_mo_previous: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_mmatom_position: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_mmatom_info: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_additional_option: global___JobInput.ImdAdditionalOption.V = ...
    @property
    def mol(self) -> global___Mol: ...
    def __init__(
        self,
        *,
        mol: typing.Optional[global___Mol] = ...,
        run: global___JobInput.RunType.V = ...,
        method: global___JobInput.MethodType.V = ...,
        basis: typing.Text = ...,
        user_options: typing.Optional[typing.Iterable[typing.Text]] = ...,
        orb1afile: typing.Text = ...,
        orb1bfile: typing.Text = ...,
        return_bond_order: builtins.bool = ...,
        xyz2: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_type: global___JobInput.ImdType.V = ...,
        imd_initial_orbital: builtins.int = ...,
        imd_orbital_type: global___JobInput.ImdOrbitalType.V = ...,
        imd_xyz_previous: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_mo_previous: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_mmatom_position: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_mmatom_info: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_additional_option: global___JobInput.ImdAdditionalOption.V = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["mol", b"mol"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "basis",
            b"basis",
            "imd_additional_option",
            b"imd_additional_option",
            "imd_initial_orbital",
            b"imd_initial_orbital",
            "imd_mmatom_info",
            b"imd_mmatom_info",
            "imd_mmatom_position",
            b"imd_mmatom_position",
            "imd_mo_previous",
            b"imd_mo_previous",
            "imd_orbital_type",
            b"imd_orbital_type",
            "imd_type",
            b"imd_type",
            "imd_xyz_previous",
            b"imd_xyz_previous",
            "method",
            b"method",
            "mol",
            b"mol",
            "orb1afile",
            b"orb1afile",
            "orb1bfile",
            b"orb1bfile",
            "return_bond_order",
            b"return_bond_order",
            "run",
            b"run",
            "user_options",
            b"user_options",
            "xyz2",
            b"xyz2",
        ],
    ) -> None: ...

global___JobInput = JobInput

class JobOutput(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor = ...
    MOL_FIELD_NUMBER: builtins.int
    ENERGY_FIELD_NUMBER: builtins.int
    GRADIENT_FIELD_NUMBER: builtins.int
    CHARGES_FIELD_NUMBER: builtins.int
    SPINS_FIELD_NUMBER: builtins.int
    DIPOLES_FIELD_NUMBER: builtins.int
    JOB_DIR_FIELD_NUMBER: builtins.int
    JOB_SCR_DIR_FIELD_NUMBER: builtins.int
    SERVER_JOB_ID_FIELD_NUMBER: builtins.int
    ORB1AFILE_FIELD_NUMBER: builtins.int
    ORB1BFILE_FIELD_NUMBER: builtins.int
    ORB_SIZE_FIELD_NUMBER: builtins.int
    BOND_ORDER_FIELD_NUMBER: builtins.int
    CI_OVERLAPS_FIELD_NUMBER: builtins.int
    CI_OVERLAP_SIZE_FIELD_NUMBER: builtins.int
    CAS_ENERGY_STATES_FIELD_NUMBER: builtins.int
    CAS_ENERGY_MULTS_FIELD_NUMBER: builtins.int
    CAS_TRANSITION_DIPOLE_FIELD_NUMBER: builtins.int
    NACME_FIELD_NUMBER: builtins.int
    ORBA_ENERGIES_FIELD_NUMBER: builtins.int
    ORBB_ENERGIES_FIELD_NUMBER: builtins.int
    ORBA_OCCUPATIONS_FIELD_NUMBER: builtins.int
    ORBB_OCCUPATIONS_FIELD_NUMBER: builtins.int
    CIS_STATES_FIELD_NUMBER: builtins.int
    CIS_UNRELAXED_DIPOLES_FIELD_NUMBER: builtins.int
    CIS_RELAXED_DIPOLES_FIELD_NUMBER: builtins.int
    CIS_TRANSITION_DIPOLES_FIELD_NUMBER: builtins.int
    CI_VEC_RE_FIELD_NUMBER: builtins.int
    CI_VEC_IM_FIELD_NUMBER: builtins.int
    COMPRESSED_BOND_ORDER_FIELD_NUMBER: builtins.int
    COMPRESSED_HESSIAN_FIELD_NUMBER: builtins.int
    COMPRESSED_AO_DATA_FIELD_NUMBER: builtins.int
    COMPRESSED_PRIMITIVE_DATA_FIELD_NUMBER: builtins.int
    COMPRESSED_MO_VECTOR_FIELD_NUMBER: builtins.int
    IMD_MMATOM_GRADIENT_FIELD_NUMBER: builtins.int
    energy: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    gradient: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    charges: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    spins: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    dipoles: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    job_dir: typing.Text = ...
    job_scr_dir: typing.Text = ...
    server_job_id: builtins.int = ...
    orb1afile: typing.Text = ...
    orb1bfile: typing.Text = ...
    orb_size: builtins.int = ...
    bond_order: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    ci_overlaps: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    ci_overlap_size: builtins.int = ...
    cas_energy_states: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.int
    ] = ...
    cas_energy_mults: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.int
    ] = ...
    cas_transition_dipole: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    nacme: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    orba_energies: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    orbb_energies: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    orba_occupations: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    orbb_occupations: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    cis_states: builtins.int = ...
    cis_unrelaxed_dipoles: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    cis_relaxed_dipoles: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    cis_transition_dipoles: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    ci_vec_re: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    ci_vec_im: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    compressed_bond_order: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.int
    ] = ...
    compressed_hessian: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    compressed_ao_data: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    compressed_primitive_data: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    compressed_mo_vector: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    imd_mmatom_gradient: google.protobuf.internal.containers.RepeatedScalarFieldContainer[
        builtins.float
    ] = ...
    @property
    def mol(self) -> global___Mol: ...
    def __init__(
        self,
        *,
        mol: typing.Optional[global___Mol] = ...,
        energy: typing.Optional[typing.Iterable[builtins.float]] = ...,
        gradient: typing.Optional[typing.Iterable[builtins.float]] = ...,
        charges: typing.Optional[typing.Iterable[builtins.float]] = ...,
        spins: typing.Optional[typing.Iterable[builtins.float]] = ...,
        dipoles: typing.Optional[typing.Iterable[builtins.float]] = ...,
        job_dir: typing.Text = ...,
        job_scr_dir: typing.Text = ...,
        server_job_id: builtins.int = ...,
        orb1afile: typing.Text = ...,
        orb1bfile: typing.Text = ...,
        orb_size: builtins.int = ...,
        bond_order: typing.Optional[typing.Iterable[builtins.float]] = ...,
        ci_overlaps: typing.Optional[typing.Iterable[builtins.float]] = ...,
        ci_overlap_size: builtins.int = ...,
        cas_energy_states: typing.Optional[typing.Iterable[builtins.int]] = ...,
        cas_energy_mults: typing.Optional[typing.Iterable[builtins.int]] = ...,
        cas_transition_dipole: typing.Optional[typing.Iterable[builtins.float]] = ...,
        nacme: typing.Optional[typing.Iterable[builtins.float]] = ...,
        orba_energies: typing.Optional[typing.Iterable[builtins.float]] = ...,
        orbb_energies: typing.Optional[typing.Iterable[builtins.float]] = ...,
        orba_occupations: typing.Optional[typing.Iterable[builtins.float]] = ...,
        orbb_occupations: typing.Optional[typing.Iterable[builtins.float]] = ...,
        cis_states: builtins.int = ...,
        cis_unrelaxed_dipoles: typing.Optional[typing.Iterable[builtins.float]] = ...,
        cis_relaxed_dipoles: typing.Optional[typing.Iterable[builtins.float]] = ...,
        cis_transition_dipoles: typing.Optional[typing.Iterable[builtins.float]] = ...,
        ci_vec_re: typing.Optional[typing.Iterable[builtins.float]] = ...,
        ci_vec_im: typing.Optional[typing.Iterable[builtins.float]] = ...,
        compressed_bond_order: typing.Optional[typing.Iterable[builtins.int]] = ...,
        compressed_hessian: typing.Optional[typing.Iterable[builtins.float]] = ...,
        compressed_ao_data: typing.Optional[typing.Iterable[builtins.float]] = ...,
        compressed_primitive_data: typing.Optional[
            typing.Iterable[builtins.float]
        ] = ...,
        compressed_mo_vector: typing.Optional[typing.Iterable[builtins.float]] = ...,
        imd_mmatom_gradient: typing.Optional[typing.Iterable[builtins.float]] = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing_extensions.Literal["mol", b"mol"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing_extensions.Literal[
            "bond_order",
            b"bond_order",
            "cas_energy_mults",
            b"cas_energy_mults",
            "cas_energy_states",
            b"cas_energy_states",
            "cas_transition_dipole",
            b"cas_transition_dipole",
            "charges",
            b"charges",
            "ci_overlap_size",
            b"ci_overlap_size",
            "ci_overlaps",
            b"ci_overlaps",
            "ci_vec_im",
            b"ci_vec_im",
            "ci_vec_re",
            b"ci_vec_re",
            "cis_relaxed_dipoles",
            b"cis_relaxed_dipoles",
            "cis_states",
            b"cis_states",
            "cis_transition_dipoles",
            b"cis_transition_dipoles",
            "cis_unrelaxed_dipoles",
            b"cis_unrelaxed_dipoles",
            "compressed_ao_data",
            b"compressed_ao_data",
            "compressed_bond_order",
            b"compressed_bond_order",
            "compressed_hessian",
            b"compressed_hessian",
            "compressed_mo_vector",
            b"compressed_mo_vector",
            "compressed_primitive_data",
            b"compressed_primitive_data",
            "dipoles",
            b"dipoles",
            "energy",
            b"energy",
            "gradient",
            b"gradient",
            "imd_mmatom_gradient",
            b"imd_mmatom_gradient",
            "job_dir",
            b"job_dir",
            "job_scr_dir",
            b"job_scr_dir",
            "mol",
            b"mol",
            "nacme",
            b"nacme",
            "orb1afile",
            b"orb1afile",
            "orb1bfile",
            b"orb1bfile",
            "orb_size",
            b"orb_size",
            "orba_energies",
            b"orba_energies",
            "orba_occupations",
            b"orba_occupations",
            "orbb_energies",
            b"orbb_energies",
            "orbb_occupations",
            b"orbb_occupations",
            "server_job_id",
            b"server_job_id",
            "spins",
            b"spins",
        ],
    ) -> None: ...

global___JobOutput = JobOutput



from pathlib import Path

import numpy as np
import pytest
import qcelemental as qcel
from qcelemental.models import AtomicInput, Molecule
from qcelemental.models.results import AtomicResult

from tcpb import terachem_server_pb2 as pb
from tcpb.clients import TCFrontEndClient, TCProtobufClient
from tcpb.config import settings
from tcpb.utils import (
    _validate_tcfe_keywords,
    atomic_input_to_job_input,
    mol_to_molecule,
)

from .conftest import _round


def test_atomic_input_to_job_input_cisco_casci_similarity(ethylene):
    """
    Test that the new atomic_input_to_job_input function produces the same protobuf
    messages that Stefan's old method created
    """
    # Dicts of options used according to Stefan's old methodology
    old_methodoloy_options = {
        "method": "hf",
        "basis": "6-31g**",
        "atoms": ethylene["atoms"],
    }
    keywords = {
        # base options
        "charge": 0,
        "spinmult": 1,
        "closed_shell": True,
        "restricted": True,
        "precision": "double",
        "convthre": 1e-8,
        "threall": 1e-20,
        # cisno options
        "cisno": "yes",
        "cisnostates": 2,
        "cisnumstates": 2,
        "closed": 7,
        "active": 2,
        "cassinglets": 2,
        "dcimaxiter": 100,
    }

    # Construct Geometry in bohr
    geom_angstrom = qcel.Datum("geometry", "angstrom", np.array(ethylene["geometry"]))
    geom_bohr = _round(geom_angstrom.to_units("bohr"))

    # Construct Molecule object
    m_ethylene = Molecule.from_data(
        {
            "symbols": ethylene["atoms"],
            "geometry": geom_bohr,
            "molecular_multiplicity": keywords["spinmult"],
            "molecular_charge": keywords["charge"],
        }
    )

    # Construct AtomicInput
    atomic_input = AtomicInput(
        molecule=m_ethylene,
        driver="energy",
        model={"method": "hf", "basis": "6-31g**"},
        keywords=keywords,
    )

    # Create protobof JobInput using Stefan's old approach
    client = TCProtobufClient("host", 11111)
    stefan_style = client._create_job_input_msg(
        "energy", geom_bohr, "bohr", **{**old_methodoloy_options, **keywords}
    )
    # Create protobuf JobInput using AtomicInput object
    job_input = atomic_input_to_job_input(atomic_input)
    assert job_input == stefan_style


def test_job_output_to_atomic_result(atomic_input, job_output):
    client = TCProtobufClient()
    atomic_result = client.job_output_to_atomic_result(
        atomic_input=atomic_input, job_output=job_output
    )
    assert isinstance(atomic_result, AtomicResult)

    # Check that all types in extras are regular python types (no longer protobuf types)
    for key, value in atomic_result.extras[settings.extras_qcvars_kwarg].items():
        assert isinstance(key, str)
        assert (
            isinstance(
                value,
                (
                    list,
                    float,
                    int,
                    str,
                    bool,
                ),
            )
            or value is None
        )


def test_job_output_to_atomic_result_correctly_sets_provenance(
    atomic_input, job_output
):
    pb_client = TCProtobufClient()
    atomic_result = pb_client.job_output_to_atomic_result(
        atomic_input=atomic_input, job_output=job_output
    )
    assert isinstance(atomic_result, AtomicResult)
    assert atomic_result.provenance.creator == pb_client.creator
    assert (
        atomic_result.provenance.routine
        == f"tcpb.{pb_client.__class__.__name__}.compute"
    )

    fe_client = TCFrontEndClient()
    atomic_result = fe_client.job_output_to_atomic_result(
        atomic_input=atomic_input, job_output=job_output
    )
    assert isinstance(atomic_result, AtomicResult)
    assert atomic_result.provenance.creator == fe_client.creator
    assert (
        atomic_result.provenance.routine
        == f"tcpb.{fe_client.__class__.__name__}.compute"
    )


def test_job_output_to_atomic_result_maintains_extras(atomic_input, job_output):
    client = TCProtobufClient()
    atomic_input.extras["mytag"] = "fake_value"
    atomic_result = client.job_output_to_atomic_result(
        atomic_input=atomic_input, job_output=job_output
    )
    assert "mytag" in atomic_result.extras


def test_mol_to_molecule_bohr():
    with open(Path(__file__).parent / "test_data" / "water_bohr.pb", "rb") as f:
        mol = pb.Mol()
        mol.ParseFromString(f.read())
    molecule = mol_to_molecule(mol)

    assert [s for s in molecule.symbols] == [a for a in mol.atoms]
    assert list(molecule.geometry.flatten()) == [coord for coord in mol.xyz]
    assert molecule.molecular_multiplicity == mol.multiplicity


def test_mol_to_molecule_angstrom():
    with open(Path(__file__).parent / "test_data" / "water_angstrom.pb", "rb") as f:
        mol = pb.Mol()
        mol.ParseFromString(f.read())
    molecule = mol_to_molecule(mol)

    geom_angstrom = qcel.Datum("geometry", "angstrom", np.array(mol.xyz))
    geom_bohr = geom_angstrom.to_units("bohr")

    assert [s for s in molecule.symbols] == [a for a in mol.atoms]
    assert _round(list(molecule.geometry.flatten())) == _round(
        [coord for coord in geom_bohr]
    )
    assert molecule.molecular_multiplicity == mol.multiplicity


def test_validate_tcfe_keywords():
    with pytest.raises(ValueError):
        _validate_tcfe_keywords({"not_good": 123})

    assert _validate_tcfe_keywords({"c0": b"123"}) is None



#!/usr/bin/env python
# Simple example showing a TDDFT calculation

import sys

from tcpb import TCProtobufClient as TCPBClient

# Ethene system
atoms = ["C", "C", "H", "H", "H", "H"]
geom = [
    0.35673483,
    -0.05087227,
    -0.47786734,
    1.61445821,
    -0.06684947,
    -0.02916681,
    -0.14997206,
    0.87780529,
    -0.62680155,
    -0.16786485,
    -0.95561368,
    -0.69426370,
    2.15270896,
    0.84221076,
    0.19314809,
    2.16553127,
    -0.97886933,
    0.15232587,
]

if len(sys.argv) != 3:
    print("Usage: {} host port".format(sys.argv[0]))
    exit(1)

with TCPBClient(host=sys.argv[1], port=int(sys.argv[2])) as TC:
    tddft_options = {
        "atoms": atoms,
        "charge": 0,
        "spinmult": 1,
        "closed_shell": True,
        "restricted": True,
        "method": "wpbeh",
        "rc_w": 0.2,
        "basis": "6-31g",
        "cis": "yes",
        "cistarget": 1,
        "cisnumstates": 3,
        "cisrelaxdipole": "yes",
    }

    # Gradient calculation
    grad_results = TC.compute_job_sync("gradient", geom, "angstrom", **tddft_options)
    print("Grad Results:\n{}".format(grad_results))

    # Coupling calculation
    nac_options = tddft_options.copy()
    nac_options["nacstate1"] = 0
    nac_options["nacstate2"] = 1
    # Try to seed as much guess as possible
    # TODO: Update this one CI and Z vectors are passed through properly
    nac_options["guess"] = "{}/{}".format(grad_results["job_scr_dir"], "c0")

    nac_results = TC.compute_job_sync("coupling", geom, "angstrom", **nac_options)
    print("\nNAC Results:\n{}".format(nac_results))



from qcelemental.models import AtomicInput, AtomicResult, FailedOperation

from tcpb import TCFrontEndClient
from tcpb.config import settings


def test_pre_compute_tasks_does_not_put_files_if_none_passed(atomic_input, mocker):
    spy = mocker.patch("tcpb.TCFrontEndClient.put")

    client = TCFrontEndClient()
    client._pre_compute_tasks(atomic_input)

    spy.assert_not_called()


def test_pre_compute_tasks_upload_c0(atomic_input, mocker):
    filename = "c0"
    filedata = b"123"

    atomic_input.extras[settings.tcfe_keywords] = {filename: filedata}

    spy = mocker.patch("tcpb.TCFrontEndClient.put")

    client = TCFrontEndClient()
    client._pre_compute_tasks(atomic_input)

    spy.assert_called_once_with(filename, filedata)


def test_pre_compute_tasks_upload_ca0_cb0(atomic_input, mocker):
    filename_a = "ca0"
    filedata_a = b"123"

    filename_b = "cb0"
    filedata_b = b"xyz"

    atomic_input.extras[settings.tcfe_keywords] = {
        filename_a: filedata_a,
        filename_b: filedata_b,
    }

    spy = mocker.patch("tcpb.TCFrontEndClient.put")

    client = TCFrontEndClient()
    client._pre_compute_tasks(atomic_input)

    assert spy.call_count == 2
    spy.assert_any_call(filename_a, filedata_a)
    spy.assert_any_call(filename_b, filedata_b)


def test_post_compute_tasks_removes_job_dir_by_default(atomic_result, mocker):
    """Implies scratch_messy and uploads_messy missing. Should cleanup directory after job"""

    # Set stdout False so tc.out not retrieved
    modified_result = atomic_result.dict()
    modified_result["protocols"]["stdout"] = False

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_called_once_with(
        "DELETE", f"{modified_result['extras'][settings.extras_job_kwarg]['job_dir']}/"
    )


def test_post_compute_tasks_retains_job_dir_is_scratch_messy(atomic_result, mocker):
    # Set stdout False so tc.out not retrieved
    modified_result = atomic_result.dict()
    modified_result["protocols"]["stdout"] = False
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_not_called()


def test_post_compute_tasks_guess_not_removed_if_not_in_uploads_dir(
    atomic_result, mocker
):
    """This tests that if a user is 'manually' passing a path to a previously computed c0
    file it does not get cleaned up by default. Only files in the uploads_dir (indicating
    that they were uploaded by the client) will get cleaned up
    """
    # Set stdout False so tc.out not retrieved
    modified_result = atomic_result.dict()
    modified_result["protocols"]["stdout"] = False

    # Set Scratch messy so no calls for removing scratch directory
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    # Set guess value NOT from a client upload (no client.uploads_prefix in path)
    client = TCFrontEndClient()
    modified_result["keywords"] = {"guess": "path/to/c0"}

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_not_called()


def test_post_compute_tasks_cleans_uploads_single_c0(atomic_result, mocker):
    # Set stdout False so tc.out not retrieved
    modified_result = atomic_result.dict()
    modified_result["protocols"]["stdout"] = False

    # Set Scratch messy so no calls for removing scratch directory
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    # Set guess value to trigger uploads cleaning
    client = TCFrontEndClient()
    path = f"{client.uploads_prefix}/path/to/c0"
    modified_result["keywords"] = {"guess": path}

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_called_once_with("DELETE", path)


def test_post_compute_tasks_cleans_uploads_ca0_cb0(atomic_result, mocker):
    # Set stdout False so tc.out not retrieved
    modified_result = atomic_result.dict()
    modified_result["protocols"]["stdout"] = False

    # Set Scratch messy so no calls for removing scratch directory
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    # Set guess value to trigger uploads cleaning
    client = TCFrontEndClient()
    patha = f"{client.uploads_prefix}/path/to/ca0"
    pathb = f"{client.uploads_prefix}/path/to/cb0"
    path = f"{patha} {pathb}"
    modified_result["keywords"] = {"guess": path}

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    assert spy.call_count == 2
    spy.assert_any_call("DELETE", patha)
    spy.assert_any_call("DELETE", pathb)


def test_post_compute_tasks_retrieves_stdout(atomic_result, mocker):
    """stdout set True be default on AtomicResultProtocols, should be retrieved by
    default"""

    # Set Scratch messy so no calls for removing scratch directory
    modified_result = atomic_result.dict()
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    stdout = b"my fake stdout"

    class fakerequest:
        text = stdout

    spy = mocker.patch("tcpb.TCFrontEndClient.get")
    spy.return_value = stdout

    client = TCFrontEndClient()
    post_compute_result = client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_called_with(
        f"{modified_result['extras'][settings.extras_job_kwarg]['job_dir']}/tc.out",
    )

    assert post_compute_result.stdout == stdout.decode()


def test_post_compute_tasks_retrieves_stdout_failed_operation(atomic_input, mocker):
    """stdout set True be default on AtomicResultProtocols, should be retrieved by
    default"""

    # Set Scratch messy so no calls for removing scratch directory
    modified_result = atomic_input.dict()
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}

    stdout = b"my fake stdout"

    class fakerequest:
        text = stdout

    spy = mocker.patch("tcpb.TCFrontEndClient.get")
    spy.return_value = stdout

    client = TCFrontEndClient()
    post_compute_result = client._post_compute_tasks(
        FailedOperation(
            input_data=AtomicInput(**modified_result),
            error={
                "error_type": "terachem_server_error",
                "error_message": "Server crashed",
            },
        )
    )
    spy.assert_called_with("/no/job/dir/tc.out")

    assert post_compute_result.error.extras["stdout"] == stdout.decode()


def test_post_compute_tasks_does_not_retrieve_stdout_or_native_files(
    atomic_result, mocker
):
    # Set Scratch messy so no calls for removing scratch directory
    modified_result = atomic_result.dict()
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}
    modified_result["protocols"]["stdout"] = False

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_not_called()


def test_post_compute_tasks_no_files_collected_native_files(atomic_result, mocker):
    # Set Scratch messy so no calls for removing scratch directory
    modified_result = atomic_result.dict()
    modified_result["extras"][settings.tcfe_keywords] = {"scratch_messy": True}
    modified_result["protocols"]["stdout"] = False
    modified_result["protocols"]["native_files"] = "none"

    spy = mocker.patch("tcpb.TCFrontEndClient._request")

    client = TCFrontEndClient()
    client._post_compute_tasks(AtomicResult(**modified_result))

    spy.assert_not_called()


def test_collect_files_queries_for_all_files_if_none_specified(atomic_result, mocker):
    modified_result = atomic_result.dict()
    modified_result["extras"][settings.extras_job_kwarg]["job_scr_dir"] = "fake_scr_dir"
    print(modified_result["extras"][settings.extras_job_kwarg])

    spy = mocker.patch("tcpb.TCFrontEndClient.ls")

    client = TCFrontEndClient()
    client._collect_files(modified_result)

    spy.assert_called_once_with("fake_scr_dir")


def test_collect_files_only_collects_specified_files_if_passed(atomic_result, mocker):
    modified_result = atomic_result.dict()
    job_scr_dir = "fake_scr_dir"
    modified_result["extras"][settings.extras_job_kwarg]["job_scr_dir"] = job_scr_dir
    filenames = ["file1", "file2"]
    modified_result["extras"][settings.tcfe_keywords] = {"native_files": filenames}

    lsspy = mocker.patch("tcpb.TCFrontEndClient.ls")
    getspy = mocker.patch("tcpb.TCFrontEndClient.get")

    client = TCFrontEndClient()
    client._collect_files(modified_result)

    lsspy.assert_not_called()
    assert getspy.call_count == 2
    getspy.assert_any_call(f"{job_scr_dir}/{filenames[0]}")
    getspy.assert_any_call(f"{job_scr_dir}/{filenames[1]}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: terachem_server.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="terachem_server.proto",
    package="terachem_server",
    syntax="proto3",
    serialized_options=b"\252\002\030Google.Protobuf.TeraChem",
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x15terachem_server.proto\x12\x0fterachem_server"\x9d\x01\n\x06Status\x12\x0c\n\x04\x62usy\x18\x01 \x01(\x08\x12\x12\n\x08\x61\x63\x63\x65pted\x18\x02 \x01(\x08H\x00\x12\x11\n\x07working\x18\x03 \x01(\x08H\x00\x12\x13\n\tcompleted\x18\x04 \x01(\x08H\x00\x12\x0f\n\x07job_dir\x18\x05 \x01(\t\x12\x13\n\x0bjob_scr_dir\x18\x06 \x01(\t\x12\x15\n\rserver_job_id\x18\x07 \x01(\x05\x42\x0c\n\njob_status"\xbd\x01\n\x03Mol\x12\r\n\x05\x61toms\x18\x01 \x03(\t\x12\x0b\n\x03xyz\x18\x02 \x03(\x01\x12,\n\x05units\x18\x03 \x01(\x0e\x32\x1d.terachem_server.Mol.UnitType\x12\x0e\n\x06\x63harge\x18\x04 \x01(\x05\x12\x14\n\x0cmultiplicity\x18\x05 \x01(\x05\x12\x0e\n\x06\x63losed\x18\x06 \x01(\x08\x12\x12\n\nrestricted\x18\x07 \x01(\x08""\n\x08UnitType\x12\x0c\n\x08\x41NGSTROM\x10\x00\x12\x08\n\x04\x42OHR\x10\x01"\xf4\t\n\x08JobInput\x12!\n\x03mol\x18\x01 \x01(\x0b\x32\x14.terachem_server.Mol\x12.\n\x03run\x18\x02 \x01(\x0e\x32!.terachem_server.JobInput.RunType\x12\x34\n\x06method\x18\x03 \x01(\x0e\x32$.terachem_server.JobInput.MethodType\x12\r\n\x05\x62\x61sis\x18\x04 \x01(\t\x12\x14\n\x0cuser_options\x18\x07 \x03(\t\x12\x11\n\torb1afile\x18\x08 \x01(\t\x12\x11\n\torb1bfile\x18\t \x01(\t\x12\x19\n\x11return_bond_order\x18\x10 \x01(\x08\x12\x0c\n\x04xyz2\x18\x11 \x03(\x01\x12\x33\n\x08imd_type\x18\x14 \x01(\x0e\x32!.terachem_server.JobInput.ImdType\x12\x1b\n\x13imd_initial_orbital\x18\x15 \x01(\x05\x12\x42\n\x10imd_orbital_type\x18\x1b \x01(\x0e\x32(.terachem_server.JobInput.ImdOrbitalType\x12\x18\n\x10imd_xyz_previous\x18\x16 \x03(\x01\x12\x17\n\x0fimd_mo_previous\x18\x17 \x03(\x02\x12\x1b\n\x13imd_mmatom_position\x18\x18 \x03(\x02\x12\x17\n\x0fimd_mmatom_info\x18\x19 \x03(\x02\x12L\n\x15imd_additional_option\x18\x1a \x01(\x0e\x32-.terachem_server.JobInput.ImdAdditionalOption"O\n\x07RunType\x12\n\n\x06\x45NERGY\x10\x00\x12\x0c\n\x08GRADIENT\x10\x01\x12\x0c\n\x08\x43OUPLING\x10\x0e\x12\x08\n\x04TDCI\x10\x10\x12\x12\n\x0e\x43I_VEC_OVERLAP\x10\x13"\xbc\x02\n\nMethodType\x12\x06\n\x02HF\x10\x00\x12\x08\n\x04\x43\x41SE\x10\x02\x12\t\n\x05SVWN1\x10\x03\x12\t\n\x05SVWN3\x10\x04\x12\t\n\x05SVWN5\x10\x05\x12\x08\n\x04SVWN\x10\x05\x12\n\n\x06\x42\x33LYP1\x10\x06\x12\t\n\x05\x42\x33LYP\x10\x06\x12\n\n\x06\x42\x33LYP3\x10\x07\x12\n\n\x06\x42\x33LYP5\x10\x08\x12\x08\n\x04\x42LYP\x10\t\x12\r\n\tBHANDHLYP\x10\n\x12\x07\n\x03PBE\x10\x0b\x12\n\n\x06REVPBE\x10\x0c\x12\x08\n\x04PBE0\x10\r\x12\x0b\n\x07REVPBE0\x10\x0e\x12\x08\n\x04WPBE\x10\x0f\x12\t\n\x05WPBEH\x10\x10\x12\x07\n\x03\x42OP\x10\x11\x12\t\n\x05MUBOP\x10\x12\x12\x0c\n\x08\x43\x41MB3LYP\x10\x13\x12\x07\n\x03\x42\x39\x37\x10\x14\x12\x08\n\x04WB97\x10\x15\x12\t\n\x05WB97X\x10\x16\x12\x0b\n\x07WB97XD3\x10\x17\x12\n\n\x06GFNXTB\x10\x18\x12\x0b\n\x07GFN2XTB\x10\x19\x1a\x02\x10\x01"P\n\x07ImdType\x12\x0b\n\x07NOT_IMD\x10\x00\x12\x15\n\x11IMD_NEW_CONDITION\x10\x01\x12\x10\n\x0cIMD_CONTINUE\x10\x02\x12\x0f\n\x0bIMD_HESSIAN\x10\x03"w\n\x0eImdOrbitalType\x12\x0e\n\nNO_ORBITAL\x10\x00\x12\x11\n\rALPHA_ORBITAL\x10\x01\x12\x10\n\x0c\x42\x45TA_ORBITAL\x10\x02\x12\x11\n\rALPHA_DENSITY\x10\x03\x12\x10\n\x0c\x42\x45TA_DENSITY\x10\x04\x12\x0b\n\x07WHOLE_C\x10\x05"C\n\x13ImdAdditionalOption\x12\x11\n\rIMD_NORMAL_MD\x10\x00\x12\x19\n\x15IMD_MECI_OPT_GRADIENT\x10\x01"\xc8\x06\n\tJobOutput\x12!\n\x03mol\x18\x01 \x01(\x0b\x32\x14.terachem_server.Mol\x12\x0e\n\x06\x65nergy\x18\x02 \x03(\x01\x12\x10\n\x08gradient\x18\x03 \x03(\x01\x12\x0f\n\x07\x63harges\x18\x04 \x03(\x01\x12\r\n\x05spins\x18\x05 \x03(\x01\x12\x0f\n\x07\x64ipoles\x18\x06 \x03(\x01\x12\x0f\n\x07job_dir\x18\t \x01(\t\x12\x13\n\x0bjob_scr_dir\x18\n \x01(\t\x12\x15\n\rserver_job_id\x18\x0b \x01(\x05\x12\x11\n\torb1afile\x18\x0c \x01(\t\x12\x11\n\torb1bfile\x18\r \x01(\t\x12\x10\n\x08orb_size\x18\x0e \x01(\x05\x12\x12\n\nbond_order\x18\x10 \x03(\x01\x12\x13\n\x0b\x63i_overlaps\x18\x11 \x03(\x01\x12\x17\n\x0f\x63i_overlap_size\x18\x12 \x01(\x05\x12\x19\n\x11\x63\x61s_energy_states\x18\x13 \x03(\x05\x12\x18\n\x10\x63\x61s_energy_mults\x18\x14 \x03(\x05\x12\x1d\n\x15\x63\x61s_transition_dipole\x18\x16 \x03(\x01\x12\r\n\x05nacme\x18\x15 \x03(\x01\x12\x15\n\rorba_energies\x18\x19 \x03(\x01\x12\x15\n\rorbb_energies\x18\x1a \x03(\x01\x12\x18\n\x10orba_occupations\x18\x1b \x03(\x01\x12\x18\n\x10orbb_occupations\x18\x1c \x03(\x01\x12\x12\n\ncis_states\x18\x1d \x01(\x05\x12\x1d\n\x15\x63is_unrelaxed_dipoles\x18\x1e \x03(\x01\x12\x1b\n\x13\x63is_relaxed_dipoles\x18\x1f \x03(\x01\x12\x1e\n\x16\x63is_transition_dipoles\x18  \x03(\x01\x12\x11\n\tci_vec_re\x18! \x03(\x01\x12\x11\n\tci_vec_im\x18" \x03(\x01\x12\x1d\n\x15\x63ompressed_bond_order\x18# \x03(\r\x12\x1a\n\x12\x63ompressed_hessian\x18$ \x03(\x02\x12\x1a\n\x12\x63ompressed_ao_data\x18% \x03(\x02\x12!\n\x19\x63ompressed_primitive_data\x18& \x03(\x02\x12\x1c\n\x14\x63ompressed_mo_vector\x18\' \x03(\x02\x12\x1b\n\x13imd_mmatom_gradient\x18( \x03(\x02*?\n\x0bMessageType\x12\n\n\x06STATUS\x10\x00\x12\x07\n\x03MOL\x10\x01\x12\x0c\n\x08JOBINPUT\x10\x02\x12\r\n\tJOBOUTPUT\x10\x03\x42\x1b\xaa\x02\x18Google.Protobuf.TeraChemb\x06proto3',
)

_MESSAGETYPE = _descriptor.EnumDescriptor(
    name="MessageType",
    full_name="terachem_server.MessageType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="STATUS",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="MOL",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="JOBINPUT",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="JOBOUTPUT",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=2508,
    serialized_end=2571,
)
_sym_db.RegisterEnumDescriptor(_MESSAGETYPE)

MessageType = enum_type_wrapper.EnumTypeWrapper(_MESSAGETYPE)
STATUS = 0
MOL = 1
JOBINPUT = 2
JOBOUTPUT = 3


_MOL_UNITTYPE = _descriptor.EnumDescriptor(
    name="UnitType",
    full_name="terachem_server.Mol.UnitType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="ANGSTROM",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BOHR",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=358,
    serialized_end=392,
)
_sym_db.RegisterEnumDescriptor(_MOL_UNITTYPE)

_JOBINPUT_RUNTYPE = _descriptor.EnumDescriptor(
    name="RunType",
    full_name="terachem_server.JobInput.RunType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="ENERGY",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="GRADIENT",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="COUPLING",
            index=2,
            number=14,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="TDCI",
            index=3,
            number=16,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="CI_VEC_OVERLAP",
            index=4,
            number=19,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=993,
    serialized_end=1072,
)
_sym_db.RegisterEnumDescriptor(_JOBINPUT_RUNTYPE)

_JOBINPUT_METHODTYPE = _descriptor.EnumDescriptor(
    name="MethodType",
    full_name="terachem_server.JobInput.MethodType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="HF",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="CASE",
            index=1,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="SVWN1",
            index=2,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="SVWN3",
            index=3,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="SVWN5",
            index=4,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="SVWN",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="B3LYP1",
            index=6,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="B3LYP",
            index=7,
            number=6,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="B3LYP3",
            index=8,
            number=7,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="B3LYP5",
            index=9,
            number=8,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BLYP",
            index=10,
            number=9,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BHANDHLYP",
            index=11,
            number=10,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="PBE",
            index=12,
            number=11,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="REVPBE",
            index=13,
            number=12,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="PBE0",
            index=14,
            number=13,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="REVPBE0",
            index=15,
            number=14,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WPBE",
            index=16,
            number=15,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WPBEH",
            index=17,
            number=16,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BOP",
            index=18,
            number=17,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="MUBOP",
            index=19,
            number=18,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="CAMB3LYP",
            index=20,
            number=19,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="B97",
            index=21,
            number=20,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WB97",
            index=22,
            number=21,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WB97X",
            index=23,
            number=22,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WB97XD3",
            index=24,
            number=23,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="GFNXTB",
            index=25,
            number=24,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="GFN2XTB",
            index=26,
            number=25,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=b"\020\001",
    serialized_start=1075,
    serialized_end=1391,
)
_sym_db.RegisterEnumDescriptor(_JOBINPUT_METHODTYPE)

_JOBINPUT_IMDTYPE = _descriptor.EnumDescriptor(
    name="ImdType",
    full_name="terachem_server.JobInput.ImdType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="NOT_IMD",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="IMD_NEW_CONDITION",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="IMD_CONTINUE",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="IMD_HESSIAN",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1393,
    serialized_end=1473,
)
_sym_db.RegisterEnumDescriptor(_JOBINPUT_IMDTYPE)

_JOBINPUT_IMDORBITALTYPE = _descriptor.EnumDescriptor(
    name="ImdOrbitalType",
    full_name="terachem_server.JobInput.ImdOrbitalType",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="NO_ORBITAL",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="ALPHA_ORBITAL",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BETA_ORBITAL",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="ALPHA_DENSITY",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="BETA_DENSITY",
            index=4,
            number=4,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="WHOLE_C",
            index=5,
            number=5,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1475,
    serialized_end=1594,
)
_sym_db.RegisterEnumDescriptor(_JOBINPUT_IMDORBITALTYPE)

_JOBINPUT_IMDADDITIONALOPTION = _descriptor.EnumDescriptor(
    name="ImdAdditionalOption",
    full_name="terachem_server.JobInput.ImdAdditionalOption",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="IMD_NORMAL_MD",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="IMD_MECI_OPT_GRADIENT",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=1596,
    serialized_end=1663,
)
_sym_db.RegisterEnumDescriptor(_JOBINPUT_IMDADDITIONALOPTION)


_STATUS = _descriptor.Descriptor(
    name="Status",
    full_name="terachem_server.Status",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="busy",
            full_name="terachem_server.Status.busy",
            index=0,
            number=1,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="accepted",
            full_name="terachem_server.Status.accepted",
            index=1,
            number=2,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="working",
            full_name="terachem_server.Status.working",
            index=2,
            number=3,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="completed",
            full_name="terachem_server.Status.completed",
            index=3,
            number=4,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="job_dir",
            full_name="terachem_server.Status.job_dir",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="job_scr_dir",
            full_name="terachem_server.Status.job_scr_dir",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="server_job_id",
            full_name="terachem_server.Status.server_job_id",
            index=6,
            number=7,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[
        _descriptor.OneofDescriptor(
            name="job_status",
            full_name="terachem_server.Status.job_status",
            index=0,
            containing_type=None,
            create_key=_descriptor._internal_create_key,
            fields=[],
        ),
    ],
    serialized_start=43,
    serialized_end=200,
)


_MOL = _descriptor.Descriptor(
    name="Mol",
    full_name="terachem_server.Mol",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="atoms",
            full_name="terachem_server.Mol.atoms",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="xyz",
            full_name="terachem_server.Mol.xyz",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="units",
            full_name="terachem_server.Mol.units",
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="charge",
            full_name="terachem_server.Mol.charge",
            index=3,
            number=4,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="multiplicity",
            full_name="terachem_server.Mol.multiplicity",
            index=4,
            number=5,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="closed",
            full_name="terachem_server.Mol.closed",
            index=5,
            number=6,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="restricted",
            full_name="terachem_server.Mol.restricted",
            index=6,
            number=7,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _MOL_UNITTYPE,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=203,
    serialized_end=392,
)


_JOBINPUT = _descriptor.Descriptor(
    name="JobInput",
    full_name="terachem_server.JobInput",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="mol",
            full_name="terachem_server.JobInput.mol",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="run",
            full_name="terachem_server.JobInput.run",
            index=1,
            number=2,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="method",
            full_name="terachem_server.JobInput.method",
            index=2,
            number=3,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="basis",
            full_name="terachem_server.JobInput.basis",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="user_options",
            full_name="terachem_server.JobInput.user_options",
            index=4,
            number=7,
            type=9,
            cpp_type=9,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orb1afile",
            full_name="terachem_server.JobInput.orb1afile",
            index=5,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orb1bfile",
            full_name="terachem_server.JobInput.orb1bfile",
            index=6,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="return_bond_order",
            full_name="terachem_server.JobInput.return_bond_order",
            index=7,
            number=16,
            type=8,
            cpp_type=7,
            label=1,
            has_default_value=False,
            default_value=False,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="xyz2",
            full_name="terachem_server.JobInput.xyz2",
            index=8,
            number=17,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_type",
            full_name="terachem_server.JobInput.imd_type",
            index=9,
            number=20,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_initial_orbital",
            full_name="terachem_server.JobInput.imd_initial_orbital",
            index=10,
            number=21,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_orbital_type",
            full_name="terachem_server.JobInput.imd_orbital_type",
            index=11,
            number=27,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_xyz_previous",
            full_name="terachem_server.JobInput.imd_xyz_previous",
            index=12,
            number=22,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_mo_previous",
            full_name="terachem_server.JobInput.imd_mo_previous",
            index=13,
            number=23,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_mmatom_position",
            full_name="terachem_server.JobInput.imd_mmatom_position",
            index=14,
            number=24,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_mmatom_info",
            full_name="terachem_server.JobInput.imd_mmatom_info",
            index=15,
            number=25,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_additional_option",
            full_name="terachem_server.JobInput.imd_additional_option",
            index=16,
            number=26,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[
        _JOBINPUT_RUNTYPE,
        _JOBINPUT_METHODTYPE,
        _JOBINPUT_IMDTYPE,
        _JOBINPUT_IMDORBITALTYPE,
        _JOBINPUT_IMDADDITIONALOPTION,
    ],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=395,
    serialized_end=1663,
)


_JOBOUTPUT = _descriptor.Descriptor(
    name="JobOutput",
    full_name="terachem_server.JobOutput",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="mol",
            full_name="terachem_server.JobOutput.mol",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=1,
            has_default_value=False,
            default_value=None,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="energy",
            full_name="terachem_server.JobOutput.energy",
            index=1,
            number=2,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="gradient",
            full_name="terachem_server.JobOutput.gradient",
            index=2,
            number=3,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="charges",
            full_name="terachem_server.JobOutput.charges",
            index=3,
            number=4,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="spins",
            full_name="terachem_server.JobOutput.spins",
            index=4,
            number=5,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="dipoles",
            full_name="terachem_server.JobOutput.dipoles",
            index=5,
            number=6,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="job_dir",
            full_name="terachem_server.JobOutput.job_dir",
            index=6,
            number=9,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="job_scr_dir",
            full_name="terachem_server.JobOutput.job_scr_dir",
            index=7,
            number=10,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="server_job_id",
            full_name="terachem_server.JobOutput.server_job_id",
            index=8,
            number=11,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orb1afile",
            full_name="terachem_server.JobOutput.orb1afile",
            index=9,
            number=12,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orb1bfile",
            full_name="terachem_server.JobOutput.orb1bfile",
            index=10,
            number=13,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orb_size",
            full_name="terachem_server.JobOutput.orb_size",
            index=11,
            number=14,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="bond_order",
            full_name="terachem_server.JobOutput.bond_order",
            index=12,
            number=16,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ci_overlaps",
            full_name="terachem_server.JobOutput.ci_overlaps",
            index=13,
            number=17,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ci_overlap_size",
            full_name="terachem_server.JobOutput.ci_overlap_size",
            index=14,
            number=18,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cas_energy_states",
            full_name="terachem_server.JobOutput.cas_energy_states",
            index=15,
            number=19,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cas_energy_mults",
            full_name="terachem_server.JobOutput.cas_energy_mults",
            index=16,
            number=20,
            type=5,
            cpp_type=1,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cas_transition_dipole",
            full_name="terachem_server.JobOutput.cas_transition_dipole",
            index=17,
            number=22,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="nacme",
            full_name="terachem_server.JobOutput.nacme",
            index=18,
            number=21,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orba_energies",
            full_name="terachem_server.JobOutput.orba_energies",
            index=19,
            number=25,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orbb_energies",
            full_name="terachem_server.JobOutput.orbb_energies",
            index=20,
            number=26,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orba_occupations",
            full_name="terachem_server.JobOutput.orba_occupations",
            index=21,
            number=27,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="orbb_occupations",
            full_name="terachem_server.JobOutput.orbb_occupations",
            index=22,
            number=28,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cis_states",
            full_name="terachem_server.JobOutput.cis_states",
            index=23,
            number=29,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cis_unrelaxed_dipoles",
            full_name="terachem_server.JobOutput.cis_unrelaxed_dipoles",
            index=24,
            number=30,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cis_relaxed_dipoles",
            full_name="terachem_server.JobOutput.cis_relaxed_dipoles",
            index=25,
            number=31,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="cis_transition_dipoles",
            full_name="terachem_server.JobOutput.cis_transition_dipoles",
            index=26,
            number=32,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ci_vec_re",
            full_name="terachem_server.JobOutput.ci_vec_re",
            index=27,
            number=33,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="ci_vec_im",
            full_name="terachem_server.JobOutput.ci_vec_im",
            index=28,
            number=34,
            type=1,
            cpp_type=5,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="compressed_bond_order",
            full_name="terachem_server.JobOutput.compressed_bond_order",
            index=29,
            number=35,
            type=13,
            cpp_type=3,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="compressed_hessian",
            full_name="terachem_server.JobOutput.compressed_hessian",
            index=30,
            number=36,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="compressed_ao_data",
            full_name="terachem_server.JobOutput.compressed_ao_data",
            index=31,
            number=37,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="compressed_primitive_data",
            full_name="terachem_server.JobOutput.compressed_primitive_data",
            index=32,
            number=38,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="compressed_mo_vector",
            full_name="terachem_server.JobOutput.compressed_mo_vector",
            index=33,
            number=39,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="imd_mmatom_gradient",
            full_name="terachem_server.JobOutput.imd_mmatom_gradient",
            index=34,
            number=40,
            type=2,
            cpp_type=6,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=1666,
    serialized_end=2506,
)

_STATUS.oneofs_by_name["job_status"].fields.append(_STATUS.fields_by_name["accepted"])
_STATUS.fields_by_name["accepted"].containing_oneof = _STATUS.oneofs_by_name[
    "job_status"
]
_STATUS.oneofs_by_name["job_status"].fields.append(_STATUS.fields_by_name["working"])
_STATUS.fields_by_name["working"].containing_oneof = _STATUS.oneofs_by_name[
    "job_status"
]
_STATUS.oneofs_by_name["job_status"].fields.append(_STATUS.fields_by_name["completed"])
_STATUS.fields_by_name["completed"].containing_oneof = _STATUS.oneofs_by_name[
    "job_status"
]
_MOL.fields_by_name["units"].enum_type = _MOL_UNITTYPE
_MOL_UNITTYPE.containing_type = _MOL
_JOBINPUT.fields_by_name["mol"].message_type = _MOL
_JOBINPUT.fields_by_name["run"].enum_type = _JOBINPUT_RUNTYPE
_JOBINPUT.fields_by_name["method"].enum_type = _JOBINPUT_METHODTYPE
_JOBINPUT.fields_by_name["imd_type"].enum_type = _JOBINPUT_IMDTYPE
_JOBINPUT.fields_by_name["imd_orbital_type"].enum_type = _JOBINPUT_IMDORBITALTYPE
_JOBINPUT.fields_by_name[
    "imd_additional_option"
].enum_type = _JOBINPUT_IMDADDITIONALOPTION
_JOBINPUT_RUNTYPE.containing_type = _JOBINPUT
_JOBINPUT_METHODTYPE.containing_type = _JOBINPUT
_JOBINPUT_IMDTYPE.containing_type = _JOBINPUT
_JOBINPUT_IMDORBITALTYPE.containing_type = _JOBINPUT
_JOBINPUT_IMDADDITIONALOPTION.containing_type = _JOBINPUT
_JOBOUTPUT.fields_by_name["mol"].message_type = _MOL
DESCRIPTOR.message_types_by_name["Status"] = _STATUS
DESCRIPTOR.message_types_by_name["Mol"] = _MOL
DESCRIPTOR.message_types_by_name["JobInput"] = _JOBINPUT
DESCRIPTOR.message_types_by_name["JobOutput"] = _JOBOUTPUT
DESCRIPTOR.enum_types_by_name["MessageType"] = _MESSAGETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Status = _reflection.GeneratedProtocolMessageType(
    "Status",
    (_message.Message,),
    {
        "DESCRIPTOR": _STATUS,
        "__module__": "terachem_server_pb2"
        # @@protoc_insertion_point(class_scope:terachem_server.Status)
    },
)
_sym_db.RegisterMessage(Status)

Mol = _reflection.GeneratedProtocolMessageType(
    "Mol",
    (_message.Message,),
    {
        "DESCRIPTOR": _MOL,
        "__module__": "terachem_server_pb2"
        # @@protoc_insertion_point(class_scope:terachem_server.Mol)
    },
)
_sym_db.RegisterMessage(Mol)

JobInput = _reflection.GeneratedProtocolMessageType(
    "JobInput",
    (_message.Message,),
    {
        "DESCRIPTOR": _JOBINPUT,
        "__module__": "terachem_server_pb2"
        # @@protoc_insertion_point(class_scope:terachem_server.JobInput)
    },
)
_sym_db.RegisterMessage(JobInput)

JobOutput = _reflection.GeneratedProtocolMessageType(
    "JobOutput",
    (_message.Message,),
    {
        "DESCRIPTOR": _JOBOUTPUT,
        "__module__": "terachem_server_pb2"
        # @@protoc_insertion_point(class_scope:terachem_server.JobOutput)
    },
)
_sym_db.RegisterMessage(JobOutput)


DESCRIPTOR._options = None
_JOBINPUT_METHODTYPE._options = None
# @@protoc_insertion_point(module_scope)



#! /usr/bin/env python
# Test of ci_vec_overlap job through TCPB

import os
import sys

from tcpb import TCProtobufClient as TCPBClient

# Ethylene system
atoms = ["C", "C", "H", "H", "H", "H"]
geom = [
    0.35673483,
    -0.05087227,
    -0.47786734,
    1.61445821,
    -0.06684947,
    -0.02916681,
    -0.14997206,
    0.87780529,
    -0.62680155,
    -0.16786485,
    -0.95561368,
    -0.69426370,
    2.15270896,
    0.84221076,
    0.19314809,
    2.16553127,
    -0.97886933,
    0.15232587,
]
geom2 = [
    0.35673483,
    -0.05087227,
    -0.47786734,
    1.61445821,
    -0.06684947,
    -0.02916681,
    -0.14997206,
    0.87780529,
    -0.62680155,
    -0.16786485,
    -0.95561368,
    -0.69426370,
    2.15270896,
    0.84221076,
    0.19314809,
    2.16553127,
    -0.97886933,
    0.15232587,
]

if len(sys.argv) != 3:
    print("Usage: {} host port".format(sys.argv[0]))
    exit(1)

with TCPBClient(host=sys.argv[1], port=int(sys.argv[2])) as TC:
    base_options = {
        "atoms": atoms,
        "charge": 0,
        "spinmult": 1,
        "closed_shell": True,
        "restricted": True,
        "method": "hf",
        "basis": "6-31g**",
        "precision": "double",
        "threall": 1.0e-20,
        "casci": "yes",
        "closed": 5,
        "active": 6,
        "cassinglets": 10,
    }

    # First run CASCI to get some test CI vectors
    casci_options = {"directci": "yes", "caswritevecs": "yes"}
    options = dict(base_options, **casci_options)
    results = TC.compute_job_sync("energy", geom, "angstrom", **options)

    # Run ci_vec_overlap job based on last job
    overlap_options = {
        "geom2": geom2,
        "cvec1file": os.path.join(results["job_scr_dir"], "CIvecs.Singlet.dat"),
        "cvec2file": os.path.join(results["job_scr_dir"], "CIvecs.Singlet.dat"),
        "orb1afile": os.path.join(results["job_scr_dir"], "c0"),
        "orb2afile": os.path.join(results["job_scr_dir"], "c0"),
    }
    options = dict(base_options, **overlap_options)
    results = TC.compute_job_sync("ci_vec_overlap", geom, "angstrom", **options)

    print("Overlap:\n{}".format(results["ci_overlap"]))



import time

import pytest
from numpy import ndarray
from qcelemental.models import AtomicInput

from tcpb import TCProtobufClient as TCPBClient


@pytest.mark.skip(
    reason="You can see in the tc.in/out files the convergence is set correctly. Yes a"
    "looser convergence doesn't always produce a shorter computation time."
)
def test_convthre(settings, atomic_input):
    atomic_input.keywords["convthre"] = 3.0e-5
    with TCPBClient(settings["tcpb_host"], settings["tcpb_port"]) as client:
        start = time.time()
        client.compute(atomic_input)
        tight_thresh = time.time() - start

    atomic_input.keywords["convthre"] = 3.0e-1
    with TCPBClient(settings["tcpb_host"], settings["tcpb_port"]) as client:
        start = time.time()
        client.compute(atomic_input)
        looser_thresh = time.time() - start

    assert tight_thresh > looser_thresh


def test_wavefunction(settings, atomic_input):
    atomic_input_dict = atomic_input.dict()
    atomic_input_dict.pop("protocols", None)
    with_wf = AtomicInput(**atomic_input_dict, protocols={"wavefunction": "all"})
    with TCPBClient(settings["tcpb_host"], settings["tcpb_port"]) as TC:
        result = TC.compute(with_wf)

    # Restricted
    assert result.wavefunction is not None
    assert isinstance(result.wavefunction.scf_eigenvalues_a, ndarray)
    assert isinstance(result.wavefunction.scf_occupations_a, ndarray)
    assert result.wavefunction.scf_eigenvalues_b is None
    assert result.wavefunction.scf_occupations_b is None

    atomic_input_dict["keywords"]["restricted"] = False
    with_wf = AtomicInput(**atomic_input_dict, protocols={"wavefunction": "all"})
    with TCPBClient(settings["tcpb_host"], settings["tcpb_port"]) as TC:
        result = TC.compute(with_wf)

    # B occupations since restricted=False
    assert result.wavefunction is not None
    assert isinstance(result.wavefunction.scf_eigenvalues_a, ndarray)
    assert isinstance(result.wavefunction.scf_occupations_a, ndarray)
    assert isinstance(result.wavefunction.scf_eigenvalues_b, ndarray)
    assert isinstance(result.wavefunction.scf_occupations_b, ndarray)



#!/usr/bin/env python
# Basic energy calculation
import sys

from qcelemental.models import AtomicInput, Molecule

from tcpb import TCProtobufClient as TCPBClient

if len(sys.argv) != 3:
    print("Usage: {} host port\n".format(sys.argv[0]))
    exit(1)


# Water system
atoms = ["O", "H", "H"]
geom = [0.0, 0.0, 0.0, 0.0, 1.5, 0.0, 0.0, 0.0, 1.5]  # in bohr

molecule = Molecule(symbols=atoms, geometry=geom)
atomic_input = AtomicInput(
    molecule=molecule,
    model={
        "method": "b3lyp",
        "basis": "6-31g",
    },
    driver="energy",
    protocols={"wavefunction": "all"},
)

with TCPBClient(host=sys.argv[1], port=int(sys.argv[2])) as client:
    result = client.compute(atomic_input)

print(result)
print(result.return_result)



#!/usr/bin/env python
# Basic energy calculation
import sys
from pprint import pprint

from qcelemental.models import AtomicInput, Molecule

from tcpb import TCFrontEndClient
from tcpb.config import settings

if len(sys.argv) != 3:
    print(f"Usage: {sys.argv[0]} host port")
    exit(1)


# Water system
atoms = ["O", "H", "H"]
geom = [0.0, 0.0, 0.0, 0.0, 1.5, 0.0, 0.0, 0.0, 1.5]  # in bohr

molecule = Molecule(symbols=atoms, geometry=geom)
atomic_input = AtomicInput(
    molecule=molecule,
    model={
        "method": "b3lyp",
        "basis": "6-31g",
    },
    # Density matrix purification appears buggy and messes with initial guess
    keywords={"purify": "no"},
    driver="energy",
    protocols={"wavefunction": "all", "stdout": True, "native_files": "all"},
)

with TCFrontEndClient(host=sys.argv[1], port=int(sys.argv[2])) as client:
    result = client.compute(atomic_input)

atomic_input_2 = atomic_input.dict()
atomic_input_2["extras"] = {settings.tcfe_keywords: {}}
atomic_input_2["extras"][settings.tcfe_keywords]["c0"] = result.native_files["c0"]
atomic_input_2["extras"][settings.tcfe_keywords]["uploads_messy"] = False

with TCFrontEndClient(host=sys.argv[1], port=int(sys.argv[2])) as client:
    result2 = client.compute(AtomicInput(**atomic_input_2))

print(result)
print(result.return_result)

pprint(result.stdout)
pprint(result2.stdout)



from __future__ import absolute_import, division, print_function

"""Exception handling for tcpb package

See https://docs.python.org/3/tutorial/errors.html#user-defined-exceptions
"""


class TCPBError(Exception):
    """Base error for package"""


class ServerError(TCPBError):
    """Raised when socket connection to server dies

    Pulls logfile out of scratch dir for convenience
    """

    def __init__(self, msg, client):
        msg += "\n\nServer Address: {}\n".format((client.host, client.port))

        job_dir = client.curr_job_dir
        job_id = client.curr_job_id
        try:
            logfile = "{}/{}.log".format(job_dir, job_id)
            with open(logfile, "r") as logf:
                lines = logf.readlines()

            debuglines = "".join(lines[-10:])
            msg += "Last 10 lines from logfile ({}):\n{}".format(logfile, debuglines)
        except IOError:
            msg += "Could not open logfile"

        super(ServerError, self).__init__(msg)



#!/usr/bin/env python
# Guess reuse example

import os
import sys

from tcpb import TCProtobufClient

# Water system
atoms = ["O", "H", "H"]
geom = [
    0.00000,
    0.00000,
    -0.06852,
    0.00000,
    -0.79069,
    0.54370,
    0.00000,
    0.79069,
    0.54370,
]
# Default geom is bohr, but this in angstrom

if len(sys.argv) != 3:
    print("Usage: {} host port".format(sys.argv[0]))
    exit(1)

with TCProtobufClient(host=sys.argv[1], port=int(sys.argv[2])) as TC:
    tc_opts = {
        "atoms": atoms,
        "charge": 0,
        "spinmult": 1,
        "closed_shell": True,
        "restricted": True,
        "method": "pbe0",
        "basis": "6-31g",
        # necessary to avoid density matrix purification on second calc which will
        # cause second calc to begin at different starting point than where first cals
        # ended
        "purify": "no",
    }

    results = TC.compute_job_sync("gradient", geom, "bohr", **tc_opts)

    # We can pull the orbital path from the previous job to feed in as a guess
    orb_path = os.path.join(results["job_scr_dir"], "c0")
    results = TC.compute_job_sync("gradient", geom, "bohr", guess=orb_path, **tc_opts)

    # Things look slightly different for unrestricted guesses
    tc_opts["closed_shell"] = False
    tc_opts["restricted"] = False
    results = TC.compute_job_sync("gradient", geom, "angstrom", **tc_opts)

    # TeraChem expects "guess <ca0 file> <cb0 file>"
    orb_paths = "{0}/ca0 {0}/cb0".format(results["job_scr_dir"])
    print(orb_paths)
    results = TC.compute_job_sync(
        "gradient", geom, "angstrom", guess=orb_paths, **tc_opts
    )



#!/usr/bin/env python
# Basic TCProtobufClient usage
import sys

from tcpb import TCProtobufClient

# Water system
atoms = ["O", "H", "H"]
geom = [
    0.00000,
    0.00000,
    -0.06852,
    0.00000,
    -0.79069,
    0.54370,
    0.00000,
    0.79069,
    0.54370,
]
# Default geom is bohr, but this in angstrom

if len(sys.argv) != 3:
    print("Usage: {} host port".format(sys.argv[0]))
    exit(1)

# Set up client for h2o job
TC = TCProtobufClient(host=sys.argv[1], port=int(sys.argv[2]))

tc_opts = {
    "atoms": atoms,
    "charge": 0,
    "spinmult": 1,
    "closed_shell": True,
    "restricted": True,
    "method": "pbe0",
    "basis": "6-31g",
}

TC.connect()

# Check if the server is available
avail = TC.is_available()
print("TCPB Server available: {}".format(avail))

# Energy calculation
# energy = TC.compute_energy(geom, "angstrom", **tc_opts)  # Default is BOHR
# print("H2O Energy: {}".format(energy))

# Gradient calculation
# energy, gradient = TC.compute_gradient(geom, "angstrom", **tc_opts)
result = TC.compute_job_sync("gradient", geom, "angstrom", **tc_opts)
print(result)
print("H2O Gradient:\n{}".format(result["gradient"]))

# # Forces calculation (just like gradient call with -1*gradient)
# energy, forces = TC.compute_forces(geom, "angstrom", **tc_opts)
# print("H2O Forces:\n{}".format(forces))

# # General calculation
# results = TC.compute_job_sync("gradient", geom, "angstrom", **tc_opts)
# print("H2O Results:\n{}".format(results))

# # Can get information from last calculation
# print("Last H2O Energy: {}".format(TC.prev_results["energy"]))

TC.disconnect()



from __future__ import absolute_import, division, print_function

"""Some small utilities for handling TeraChem serialized files
"""

import numpy as np


def read_orbfile(orbfile, num_rows, num_cols):
    """Deserialize a TeraChem binary orbital file of doubles.

    HF/DFT orbitals (which are stored column-major for TeraChem) are transposed on deserialization.

    Args:
        orbfile: Filename of orbital file to read
        num_rows: Rows in MO coefficient matrix
        num_cols: Columns in MO coefficient matrix

    Returns:
        (num_rows, num_cols): NumPy array of MO coefficients
    """
    orbs = np.fromfile(orbfile, dtype=np.float64)

    orbs = orbs.reshape((num_rows, num_cols))

    if orbfile.endswith("c0") or orbfile.endswith("ca0") or orbfile.endswith("cb0"):
        orbs = orbs.transpose()

    return orbs


def write_orbfile(orbs, orbfile):
    """Serialize a TeraChem binary orbital file of doubles.

    HF/DFT orbitals (which are stored column-major for TeraChem) are transposed on serialization.

    Args:
        orbs: Non-flat NumPy array of MO coefficients
        orbfile: Filename of orbital file to write
    """
    if not isinstance(orbs, np.ndarray) or len(orbs.shape) != 2:
        raise SyntaxError(
            "Need a shaped NumPy array for write_orbfile to do proper serialization for TeraChem."
        )

    if orbfile.endswith("c0") or orbfile.endswith("ca0") or orbfile.endswith("cb0"):
        orbs = orbs.transpose()

    orbs.astype(np.float64).tofile(orbfile)


def read_ci_vector(cvecfile, num_rows, num_cols):
    """Deserialize a TeraChem binary CI vector file of doubles.

    Args:
        cvecfile: Filename of CI vector file to read
        num_rows: Rows in CI vector
        num_cols: Columns in CI vector matrix
    Returns a (num_rows, num_cols) NumPy array of MO coefficients
    """
    ci_vector = np.fromfile(cvecfile, dtype=np.float64)

    return ci_vector.reshape((num_rows, num_cols))


# NOTE: Commented out because it appears unfunctional. Looks for variable "orbs"
# but it is never declared. Preserving function for now...
# def write_ci_vector(ci_vector, orbfile):
#     """Serialize a TeraChem binary CI vector file of doubles.

#     Args:
#         ci_vector: Non-flat NumPy array of CI vector
#         cvecfile: Filename of CI vector file to write
#     """
#     if not isinstance(orbs, np.ndarray) or len(orbs.shape) != 2:
#         raise SyntaxError(
#             "Need a shaped NumPy array for write_cvecfile to do proper serialization for TeraChem."
#         )

#     ci_vector.astype(np.float64).tofile(orbfile)
